%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.1, (the "License"); you may not use this file except in
%% compliance with the License. You should have received a copy of the
%% Erlang Public License along with this software. If not, it can be
%% retrieved via the world wide web at http://www.erlang.org/.
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Initial Developer of the Original Code is Ericsson Utvecklings AB.
%% Portions created by Ericsson are Copyright 1999, Ericsson Utvecklings
%% AB. All Rights Reserved.''
%% 
%%     $Id$
%%
-module(beam_literals_SUITE).
-export([all/1]).
-export([putting/1, matching_smalls/1, matching_smalls_jt/1,
	 matching_bigs/1, matching_more_bigs/1,
	 matching_bigs_and_smalls/1, badmatch/1,
	 receiving/1, literal_type_tests/1]).

-include("test_server.hrl").

all(suite) -> [putting, matching_smalls, matching_smalls_jt,
	       matching_bigs, matching_more_bigs,
	       matching_bigs_and_smalls, badmatch, receiving, literal_type_tests].

putting(doc) -> "Test creating lists and tuples containing big number literals.";
putting(suite) -> [];
putting(Config) when list(Config) ->
    -773973888575940946173283 = chksum(putting1(8987697898797)).

putting1(X) ->
    {8797987987987987872256443, [1324483773773], {3.1415, 2.71, [2.5, 35.125|9.31]},
     [X|349873987387373],
     [329878349873|-387394729872], -773973937933873929749873}.

matching_bigs(doc) -> "Test matching of a few big number literals (in Beam,"
		      "select_val/3 will NOT be used).";
matching_bigs(suite) -> [];
matching_bigs(Config) when list(Config) ->
    a = matching1(3972907842873739),
    b = matching1(-389789298378939783333333333333333333784),
    other = matching1(42).

matching_smalls(doc) -> "Test matching small numbers (both positive and negative).";
matching_smalls(suite) -> [];
matching_smalls(Config) when list(Config) ->
    ?line a = m_small(-42),
    ?line b = m_small(0),
    ?line c = m_small(105),
    ?line d = m_small(-13),
    ?line e = m_small(337848),
    ?line other = m_small(324),
    ?line other = m_small(-7),
    ok.

m_small(-42) -> a;
m_small(0) -> b;
m_small(105) -> c;
m_small(-13) -> d;
m_small(337848) -> e;
m_small(_) -> other.

matching_smalls_jt(doc) ->
    "Test matching small numbers (both positive and negative). "
	"Make sure that a jump table is used.";
matching_smalls_jt(suite) -> [];
matching_smalls_jt(Config) when list(Config) ->
    ?line a = m_small_jt(-2),
    ?line b = m_small_jt(-1),
    ?line c = m_small_jt(0),
    ?line d = m_small_jt(2),
    ?line e = m_small_jt(3),
    ?line other = m_small(324),
    ?line other = m_small(-7),
    ok.

m_small_jt(-2) -> a;
m_small_jt(-1) -> b;
m_small_jt(0) -> c;
m_small_jt(2) -> d;
m_small_jt(3) -> e;
m_small_jt(_) -> other.

%% Big numbers, no select_val.

matching1(3972907842873739) -> a;
matching1(-389789298378939783333333333333333333784) -> b;
matching1(_) -> other.


matching_more_bigs(doc) -> "Test matching of a big number literals (in Beam,"
		      "a select_val/3 instruction will be used).";
matching_more_bigs(suite) -> [];
matching_more_bigs(Config) when list(Config) ->
    a = matching2(-999766349740978337),
    b = matching2(9734097866575478),
    c = matching2(-966394677364879734),
    d = matching2(13987294872948990),
    e = matching2(777723896192459245),
    other = matching2(7),
    other = matching2(39789827988888888888888888888347474444444444444444444).

%% Big numbers with select_val.

matching2(-999766349740978337) -> a;
matching2(9734097866575478) -> b;
matching2(-966394677364879734) -> c;
matching2(13987294872948990) -> d;
matching2(777723896192459245) -> e;
matching2(_) -> other.

matching_bigs_and_smalls(doc) -> "Test matching of a mix of big numbers and literals.";
matching_bigs_and_smalls(suite) -> [];
matching_bigs_and_smalls(Config) when list(Config) ->
    a = matching3(38472928723987239873873),
    b = matching3(0),
    c = matching3(-3873973932710954671207461057614287561348756348743634876436784367873),
    d = matching3(3978429867297393873),
    e = matching3(42),
    f = matching3(-4533),
    other = matching3(77),
    other = matching3(39274120984379249874219748).

%% Mixed small and big.

matching3(38472928723987239873873) -> a;
matching3(0) -> b;
matching3(-3873973932710954671207461057614287561348756348743634876436784367873) -> c;
matching3(3978429867297393873) -> d;
matching3(42) -> e;
matching3(-4533) -> f;
matching3(_) -> other.

badmatch(doc) -> "Test literal badmatches with big number and floats.";
badmatch(suite) -> [];
badmatch(Config) when list(Config) ->
    %% We are satisfied if we can load this module and run it.
    Big = 32984798729847892498297824872982972978239874,
    Float = 3.1415927,
    ?line catch a = Big,
    ?line catch b = Float,
    ?line case Big of
	      Big -> ok
	  end,
    ?line case Float of
	      Float -> ok
	  end,
    ok.

receiving(doc) -> "Test receive with a big number literal (more than 27 bits, "
		      "less than 32 bits).";
receiving(suite) -> [];
receiving(Config) when list(Config) ->
    Self = self(),
    fun_spawn(fun() -> Self ! here_is_a_message end),
    ok = receive
	     here_is_a_message ->
		 ok
	 after 16#f1234567 ->
		 timeout
	 end.

literal_type_tests(doc) -> "Test type tests on literal values.";
literal_type_tests(suite) -> [];
literal_type_tests(Config) when list(Config) ->
    %% Generate an Erlang module with all different type of type tests.
    ?line Tests = make_test([{T, L} || T <- type_tests(), L <- literals()]),
    ?line Mod = literal_test,
    ?line Func = {function, 0, test, 0, [{clause,0,[],[],Tests}]},
    ?line Form = [{attribute,0,module,Mod},
		  {attribute,0,compile,export_all},
		  Func, {eof,0}],

    %% Print generated code for inspection.
    ?line lists:foreach(fun (F) -> io:put_chars([erl_pp:form(F),"\n"]) end, Form),

    %% Test compile:form/1.  This implies full optimization (default).
    ?line {ok,Mod,Code1} = compile:forms(Form),
    ?line {module,Mod} = code:load_binary(Mod, Mod, Code1),
    ?line Mod:test(),
    ?line true = code:delete(Mod),
    ?line code:purge(Mod),
			       
    %% Test compile:form/2.  Turn off all optimizations (for the new compiler).
    ?line {ok,Mod,Code2} = compile:forms(Form, [binary, no_opt, no_kernopt, no_postopt]),
    ?line {module,Mod} = code:load_binary(Mod, Mod, Code2),
    ?line Mod:test(),
    ?line true = code:delete(Mod),
    ?line code:purge(Mod),
    ok.

make_test([{T, L}|Ts]) ->
    [test(T, L)|make_test(Ts)];
make_test([]) ->
    [].

test(T, L) ->
    S = lists:flatten(io_lib:format("begin io:format(\"~~p~~n\", [{~p,~p}]), if ~w(~w) -> true; true -> false end end. ", [T, L, T, L])),
    {ok, Toks, Line} = erl_scan:string(S),
    {ok, E} = erl_parse:parse_exprs(Toks),
    {value,Val,Bs} = erl_eval:exprs(E, []),
    {match,0,{atom,0,Val},hd(E)}.
    
literals() ->
    [42,
     3.14,
     -3,
     32982724987789283473473838474,
     [],
     xxxx].

type_tests() ->
    [integer,
     float,
     number,
     atom,
     constant,
     list,
     tuple,
     pid,
     reference,
     port,
     binary,
     function].
    
%% Help functions.

chksum(Term) ->
    chksum(Term, 0).
chksum([List|T], Sum) when list(List) ->
    chksum(T, chksum(List, Sum));
chksum([H|T], Sum) ->
    chksum(T, chksum(H, Sum));
chksum([], Sum) ->
    Sum;
chksum(Tuple, Sum) when tuple(Tuple) ->
    chksum(tuple_to_list(Tuple), Sum);
chksum(Int, Sum) when integer(Int) ->
    Sum * 5 + Int;
chksum(Other, Sum) ->
    erlang:hash(Other, 39729747).

fun_spawn(Fun) ->
    fun_spawn(Fun, []).

fun_spawn(Fun, Args) ->
    spawn_link(erlang, apply, [Fun, Args]).
